package net.ipksindia
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import net.ipksindia.config.AppConfig
import net.ipksindia.model.NeftTransaction
import net.ipksindia.model.TransferTransaction
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.RequestBody.Companion.toRequestBody
import org.json.JSONException
import org.json.JSONObject
import response.TransactionFailureResponse
import response.TransactionResponse
import response.TransactionSuccessResponse
import java.io.IOException


class TransactionExecutor() {

    private val transactionUrl = AppConfig.remoteServerConfig.transactionUrl

    fun executePair(transactionPair: Pair<TransferTransaction, NeftTransaction>): Pair<TransactionResponse, TransactionResponse> {

        val transferTransaction = transactionPair.first
        val neftTransaction = transactionPair.second

        val transferResponse = execute(Json.encodeToString(transferTransaction))
        val neftResponse = execute(Json.encodeToString(neftTransaction))
        return Pair(transferResponse, neftResponse)
    }

    private fun execute(postBody: String): TransactionResponse {
//        println(postBody)
        val jsonMediaType = "application/json; charset=utf-8".toMediaType()

        val httpClient = OkHttpClient
            .Builder()
            .hostnameVerifier { _, _ -> true }
            .build()

        val request = Request
            .Builder()
            .url(transactionUrl)
            .post(postBody.toRequestBody(jsonMediaType))
            .build()

        val responseBody = httpClient.newCall(request).execute().use { response ->
            if (!response.isSuccessful) {
                throw IOException("Unexpected response: ${response.body}")
            }
            response.body?.string() ?: throw IOException("no response body")
        }

        val responseObj = JSONObject(responseBody)
        val status = try {responseObj.getString("status") } catch(_: JSONException) { "" }
        val message = try { responseObj.getString("message") }catch(_: JSONException) { "" }
        val error = try { responseObj.getInt("error") } catch(_: JSONException) { 1 }

        if(responseBody.contains("SUCCESS")) {
            val queueNo = try { responseObj.getJSONObject("response").getString("QueueId") } catch(_: JSONException) { "" }
            return TransactionSuccessResponse(status, message, queueNo, error)
        } else {
            val errorMsg = try { responseObj.getJSONObject("response").getString("errorMsg") } catch(_: JSONException) { responseBody }
            return TransactionFailureResponse(status, message, errorMsg, error)
        }

    }

}